@@ -239,44 +239,73 @@ static gint
 gail_expander_get_n_children (AtkObject* obj)
 {
   GtkWidget *widget;
-  GtkWidget *child;
+  GList *children;
+  gint count = 0;
+
+  g_return_val_if_fail (GAIL_IS_CONTAINER (obj), count);
 
   widget = GTK_ACCESSIBLE (obj)->widget;
-
   if (widget == NULL)
     return 0;
 
-  child = GTK_BIN (widget)->child;
-  if (child && !GTK_IS_LABEL (child))
-    return 1; 
+  children = gtk_container_get_children (GTK_CONTAINER(widget));
+  count = g_list_length (children);
+  g_list_free (children);
 
-  return 0;
+  /* See if there is a label - if there is, reduce our count by 1
+   * since we don't want the label included with the children.
+   */
+  if (gtk_expander_get_label_widget (widget))
+    count -= 1;
+
+  return count; 
 }
 
 static AtkObject*
 gail_expander_ref_child (AtkObject *obj,
                          gint      i)
 {
+  GList *children, *tmp_list;
+  AtkObject *accessible;
   GtkWidget *widget;
-  GtkWidget *child;
-  AtkObject *child_obj;
+  GtkWidget *label;
+  gint index;
+  gint count;
 
+  g_return_val_if_fail (GAIL_IS_CONTAINER (obj), NULL);
+  g_return_val_if_fail ((i >= 0), NULL);
   widget = GTK_ACCESSIBLE (obj)->widget;
-
   if (widget == NULL)
     return NULL;
 
-  if (i != 0)
-    return NULL;
-    
-  child = GTK_BIN (widget)->child;
-  if (child && !GTK_IS_LABEL (child))
-    {
-      child_obj = gtk_widget_get_accessible (child);
-      return g_object_ref (child_obj);
-    }
+  children = gtk_container_get_children (GTK_CONTAINER (widget));
 
-  return NULL;
+  /* See if there is a label - if there is, we need to skip it
+   * since we don't want the label included with the children.
+   */
+  label = gtk_expander_get_label_widget (widget);
+  if (label) {
+    count = g_list_length (children);
+    for (index = 0; index <= i; index++) {
+      tmp_list = g_list_nth (children, index);
+      if (label == GTK_WIDGET (tmp_list->data)) {
+	i += 1;
+	break;
+      }
+    }
+  }
+
+  tmp_list = g_list_nth (children, i);
+  if (!tmp_list)
+    {
+      g_list_free (children);
+      return NULL;
+    }  
+  accessible = gtk_widget_get_accessible (GTK_WIDGET (tmp_list->data));
+
+  g_list_free (children);
+  g_object_ref (accessible);
+  return accessible; 
 }
 
 static void
@@ -348,6 +377,8 @@ gail_expander_real_notify_gtk (GObject    *obj,
     {
       atk_object_notify_state_change (atk_obj, ATK_STATE_CHECKED, 
                                       gtk_expander_get_expanded (expander));
+      atk_object_notify_state_change (atk_obj, ATK_STATE_EXPANDED, 
+                                      gtk_expander_get_expanded (expander));
       g_signal_emit_by_name (atk_obj, "visible_data_changed");
     }
   else
@@ -573,8 +604,12 @@ gail_expander_ref_state_set (AtkObject *obj)
 
   expander = GTK_EXPANDER (widget);
 
-  if (gtk_expander_get_expanded (expander))
+  atk_state_set_add_state (state_set, ATK_STATE_EXPANDABLE);
+
+  if (gtk_expander_get_expanded (expander)) {
     atk_state_set_add_state (state_set, ATK_STATE_CHECKED);
+    atk_state_set_add_state (state_set, ATK_STATE_EXPANDED);
+  }
 
   return state_set;
 }
